"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const core_1 = require("@collectable/core");
const internals_1 = require("../internals");
/**
 * Retrieves the last entry in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {([K, V]|undefined)} A key/value tuple for the last entry in the tree, or undefined if the tree was empty
 */
function last(tree) {
    if (tree._size === 0)
        return void 0;
    var node = tree._root;
    while (!internals_1.isNone(node._right)) {
        node = node._right;
    }
    return node;
}
exports.last = last;
/**
 * Retrieves the last key in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {([K, V]|undefined)} The key of the last entry in the tree, or undefined if the tree was empty
 */
function lastKey(tree) {
    var node = last(tree);
    return core_1.isDefined(node) ? node.key : void 0;
}
exports.lastKey = lastKey;
/**
 * Retrieves the value of the last entry in the tree.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {([K, V]|undefined)} The value of the last entry in the tree, or undefined if the tree was empty
 */
function lastValue(tree) {
    var node = last(tree);
    return core_1.isDefined(node) ? node.value : void 0;
}
exports.lastValue = lastValue;
/**
 * Returns an iterator that starts from the last entry in the tree and iterates toward the start of the tree. Emissions
 * are references to nodes in the tree, exposed directly to allow Collectable.RedBlackTree to be efficiently consumed as
 * a backing structure for other data structures. Do not modify the returned node.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {RedBlackTreeIterator<K, V>} An iterator for entries in the tree
 */
function iterateFromLast(tree) {
    var path = internals_1.PathNode.NONE;
    var node = tree._root;
    while (!internals_1.isNone(node)) {
        path = internals_1.PathNode.next(node, path, 2 /* RIGHT */);
        node = node._right;
    }
    return internals_1.RedBlackTreeIterator.create(path, tree._compare, true);
}
exports.iterateFromLast = iterateFromLast;
function iterateValuesFromLast(tree) {
    return new internals_1.RedBlackTreeValueIterator(iterateFromLast(tree));
}
exports.iterateValuesFromLast = iterateValuesFromLast;
function iterateKeysFromLast(tree) {
    return new internals_1.RedBlackTreeKeyIterator(iterateFromLast(tree));
}
exports.iterateKeysFromLast = iterateKeysFromLast;

//# sourceMappingURL=last.js.map
