import { RedBlackTreeStructure, RedBlackTreeIterator, RedBlackTreeValueIterator, RedBlackTreeKeyIterator, RedBlackTreeEntry } from '../internals';
/**
 * Retrieves the entry at the specified index (ordinal) in the tree. If a negative number is specified for the index,
 * the number is treated as a backtracking offset, with -1 matching the last element in the list, -2 matching the
 * second-last, and so forth.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {number} index The numeric index of the entry to retrieve (A negative number counts as a backtracking offset from the end of the tree)
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(RedBlackTreeEntry<K, V>|undefined)} The tree entry at the specified index, or undefined if the index was out of range
 */
export declare function at<K, V = null>(index: number, tree: RedBlackTreeStructure<K, V>): RedBlackTreeEntry<K, V> | undefined;
/**
 * Retrieves the key at the specified index (ordinal) in the tree. If a negative number is specified for the index,
 * the number is treated as a backtracking offset, with -1 matching the last element in the list, -2 matching the
 * second-last, and so forth.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {number} index The numeric index of the entry to retrieve (A negative number counts as a backtracking offset from the end of the tree)
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(K|undefined)} The key at the specified index, or undefined if the index was out of range
 */
export declare function keyAt<K, V = null>(index: number, tree: RedBlackTreeStructure<K, V>): K | undefined;
/**
 * Retrieves the value of the entry at the specified index (ordinal) in the tree. If a negative number is specified for
 * the index, the number is treated as a backtracking offset, with -1 matching the last element in the list, -2 matching
 * the second-last, and so forth.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {number} index The numeric index of the entry to retrieve (A negative number counts as a backtracking offset from the end of the tree)
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {(V|undefined)} The value at the specified index, or undefined if the index was out of range
 */
export declare function valueAt<K, V = null>(index: number, tree: RedBlackTreeStructure<K, V>): V | undefined;
/**
 * Determines the index (ordinal) of the tree entry that has the specified key. If the key does not exist in the tree, -1 is returned.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {K} key The key of the tree entry to find the index for
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {number} The index of the key in the tree, or -1 if the key was not found
 */
export declare function indexOf<K, V = null>(key: K, tree: RedBlackTreeStructure<K, V>): number;
/**
 * Creates an iterator for which the first entry is at the specified index in the tree. If the index is out of range, an
 * empty iterator is returned.
 *
 * @export
 * @template K The type of keys in the tree
 * @template V The type of values in the tree
 * @param {boolean} reverse If true, the iterator will iterate backward toward the first entry in the tree
 * @param {number} index The numeric index of the entry to retrieve (A negative number counts as a backtracking offset from the end of the tree)
 * @param {RedBlackTreeStructure<K, V>} tree The input tree
 * @returns {RedBlackTreeIterator<K, V>} An iterator that retrieves each successive entry in the tree, starting from the specified index
 */
export declare function iterateFromIndex<K, V = null>(reverse: boolean, index: number, tree: RedBlackTreeStructure<K, V>): RedBlackTreeIterator<K, V>;
export declare function iterateValuesFromIndex<K, V = null>(reverse: boolean, index: number, tree: RedBlackTreeStructure<K, V>): RedBlackTreeValueIterator<K, V>;
export declare function iterateKeysFromIndex<K, V = null>(reverse: boolean, index: number, tree: RedBlackTreeStructure<K, V>): RedBlackTreeKeyIterator<K, V>;
